
//*****************************************************************************
//
// File Name	: 'a2d.c'
// Title			: Analog-to-digital converter functions
// Author		: Pascal Stang - Copyright (C) 2002
// Created		: 4/08/2002
// Revised		: 4/30/2002
// Version		: 1.1
// Target MCU	: Atmel AVR series
// Editor Tabs	: 3
//
// This code is distributed under the GNU Public License
//		which can be found at http://www.gnu.org/licenses/gpl.txt
//
//*****************************************************************************

#include <io.h>
#include <sig-avr.h>
#include <interrupt.h>

#include "global.h"
#include "a2d.h"

// global variables

// software flag used to indicate when
// the conversion is complete
volatile unsigned char a2dCompleteFlag;

// interrupt handler for ADC complete interrupt
SIGNAL(SIG_ADC)
{
	// set the a2d conversion flag
	// to indicate "complete"
	a2dCompleteFlag = TRUE;
}

// functions
void a2dInit(void)
{
	sbi(ADCSR, ADEN);						// enable ADC (turn on ADC power)
	cbi(ADCSR, ADFR);						// default to single sample convert mode
	a2dSetPrescaler(ADC_PRESCALE);	// set default prescaler
	a2dSetReference(ADC_REFERENCE);	// set default reference

   sbi(ADCSR, ADIE);						// enable ADC interrupts
		
	a2dCompleteFlag = FALSE;			// clear conversion complete flag
	sei();									// turn on interrupts (if not already on)
}

void a2dOff(void)
{
	cbi(ADCSR, ADIE);						// disable ADC interrupts
	cbi(ADCSR, ADEN);						// disable ADC (turn off ADC power)
}

void a2dSetPrescaler(unsigned char prescale)
{
	// configure A2D converter clock division (prescaling)
	outp( ((inp(ADCSR) & ~ADC_PRESCALE_MASK) | prescale), ADCSR);
}

void a2dSetReference(unsigned char ref)
{
	// configure A2D converter voltage reference
	outp( ((inp(ADMUX) & ~ADC_REFERENCE_MASK) | (ref<<6)), ADMUX);
}

unsigned short a2dConvert10bit(unsigned char ch)
{
	a2dCompleteFlag = FALSE;	// clear conversion complete flag
	outp((inp(ADMUX) & ~ADC_MUX_MASK) | (ch & ADC_MUX_MASK), ADMUX);	// set channel
	sbi(ADCSR, ADIF);				// clear hardware "conversion complete" flag 
	sbi(ADCSR, ADSC);				// start conversion
	while(!a2dCompleteFlag);	// wait until conversion complete

	// CAUTION: MUST READ ADCL BEFORE ADCH!!!
	return (inp(ADCL) | (inp(ADCH)<<8));	// read ADC (full 10 bits);
}

unsigned char a2dConvert8bit(unsigned char ch)
{
	a2dCompleteFlag = FALSE;	// clear conversion complete flag
	outp((inp(ADMUX) & ~ADC_MUX_MASK) | (ch & ADC_MUX_MASK), ADMUX);	// set channel
	sbi(ADCSR, ADIF);				// clear hardware "conversion complete" flag
	sbi(ADCSR, ADSC);				// start conversion
	asm volatile ("nop");
	while(!a2dCompleteFlag);	// wait until conversion complete
	//while( bit_is_clear(ADCSR, ADIF) );	// wait until conversion complete

	// CAUTION: MUST READ ADCL BEFORE ADCH!!!
	return (((inp(ADCL)>>2 & 0x3F) | (inp(ADCH)<<6 & 0xC0)));	// return ADC MSB byte
}

